package gonganxitong

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/web/html"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type member struct {
	Tables
	user    user
	order   string `data:"102"`
	role    string `data:"leader,worker,server"`
	remove  string `name:"remove" role:"void"`
	enable  string `name:"enable" role:"void"`
	disable string `name:"disable" role:"void"`
	promote string `name:"promote" role:"void"`
	demote  string `name:"demote" role:"void"`
}

func (s member) Remove(m *ice.Message, arg ...string) {
	if s.IsLeader(m) {
		m.Cmdy(s.UserPlace, s.Delete, s.option(m, model.ROLE, kit.Format(UserPlaceTenant), model.STATUS, kit.Format(MemberDisabled)))
		s.recordEvent(m, "❌", m.Trans("remove member", "删除成员"), html.DANGER)
	}
}
func (s member) Disable(m *ice.Message, arg ...string) {
	if s.IsLeader(m) {
		m.Cmdy(s.UserPlace, s.UpdateField, model.STATUS, MemberDisabled, s.option(m, model.ROLE, kit.Format(UserPlaceTenant)))
		s.recordEvent(m, "❌", m.Trans("disable member", "禁用成员"), html.DANGER)
	}
}
func (s member) Demote(m *ice.Message, arg ...string) {
	if s.IsCreator(m) {
		m.Cmdy(s.UserPlace, s.UpdateField, model.ROLE, kit.Format(UserPlaceTenant), s.option(m))
		s.recordEvent(m, "❌", m.Trans("demote worker", "降级成为工作人员"), html.DANGER)
	}
}
func (s member) Promote(m *ice.Message, arg ...string) {
	if s.IsCreator(m) {
		m.Cmdy(s.UserPlace, s.UpdateField, model.ROLE, kit.Format(UserPlaceLandlord), s.option(m))
		s.recordEvent(m, "✅", m.Trans("promote leader", "升级成为管理人员"), html.NOTICE)
	}
}
func (s member) Enable(m *ice.Message, arg ...string) {
	if s.IsLeader(m) {
		m.Cmdy(s.UserPlace, s.UpdateField, model.STATUS, MemberNormal, s.option(m, model.ROLE, kit.Format(UserPlaceTenant)))
		s.recordEvent(m, "✅", m.Trans("enable member", "启用成员"), html.NOTICE)
	}
}
func (s member) List(m *ice.Message, arg ...string) {
	USER_PLACE_ROLE := s.Keys(s.UserPlace, model.ROLE)
	s.Limit(m, 1000)
	user_uid, isLeader, isCreator := m.Option(model.USER_UID), s.IsLeader(m), s.IsCreator(m)
	s.FieldsWithCreatedAT(m, s.UserPlace, model.USER_UID, s.Keys(s.UserPlace, model.ROLE), s.AS(s.Key(s.UserPlace, model.STATUS), model.MEMBER_STATUS))
	if len(arg) == 1 {
		m.Cmdy(s.UserPlace, s.Select, s.Keys(s.Place, model.UID), arg[0]).Action()
	} else if len(arg) == 2 {
		m.FieldsSetDetail().Cmdy(s.UserPlace, s.Select, s.Keys(s.Place, model.UID), arg[0], s.Key(s.UserPlace, model.UID), arg[1]).Action()
	} else {
		return
	}
	defer m.Sort(kit.Fields(model.MEMBER_STATUS, USER_PLACE_ROLE, "auth_status", model.CREATED_AT))
	m.Table(func(value ice.Maps) {
		if !m.FieldsIsDetail() {
			if user_uid == value[model.USER_UID] {
				m.Push("who", "myself")
			} else {
				m.Push("who", "")
			}
		}
		button := []ice.Any{}
		if MemberStatus(kit.Int(value[model.MEMBER_STATUS])) == MemberNormal {
			if isCreator {
				switch kit.Int(value[USER_PLACE_ROLE]) {
				case 2:
					button = append(button, s.Demote)
				case 3:
					button = append(button, s.Promote)
				}
			}
			if isLeader {
				switch kit.Int(value[USER_PLACE_ROLE]) {
				case 3:
					button = append(button, s.Disable)
				}
			}
		} else {
			if isCreator {
				button = append(button, s.Enable, s.Remove)
			} else if isLeader {
				switch kit.Int(value[USER_PLACE_ROLE]) {
				case 3:
					button = append(button, s.Enable, s.Remove)
				}
			}
		}
		m.PushButton(button...)
	})
	s.SelectJoinUser(m, model.NAME, model.INFO, model.AVATAR, model.AUTH_UID)
	s.SelectJoinAuth(m)
	s.DisplayBase(m, "").DisplayCSS("")
	m.Echo("count: %d", m.Length())
}

func init() { ice.TeamCtxCmd(member{Tables: newTables()}) }

func (s member) option(m *ice.Message, arg ...string) []string {
	return kit.Simple(m.OptionSimple(s.Keys(s.Place, model.UID), model.UID), arg)
}
func (s member) recordEvent(m *ice.Message, icon, title, style string, arg ...string) {
	s.RecordEvent(m, kit.JoinWord(icon, title, kit.Cut(m.Option(model.UID), 6), s.TransRole(m, "", style), m.Option(model.USER_NAME)), m.Option(model.UID))
}

type MemberStatus int

const (
	MemberNormal MemberStatus = iota
	MemberDisabled
)

var MemberStatusList = map[MemberStatus]string{
	MemberNormal:   "normal",
	MemberDisabled: "disabled",
}

func (s MemberStatus) String() string { return MemberStatusList[s] }

type AuthStatus int

const (
	AuthCreate AuthStatus = iota
	AuthRequest
	AuthIssued
	AuthRevoked
)

var AuthStatusList = map[AuthStatus]string{
	AuthCreate:  "create",
	AuthRequest: "request",
	AuthIssued:  "issued",
	AuthRevoked: "revoked",
}

func (s AuthStatus) String() string { return AuthStatusList[s] }
